@@ -18,24 +18,20 @@ class Linear(nn.Linear):
                  weight_bit_width=8,
                  bias_bit_width=16,
                  inter_bit_width=32,
-                 acti_bit_width=8,
                  retrain=True,
                  quant=False):
         super().__init__(in_features, out_features, bias=bias)
         self.weight_bit_width = weight_bit_width
         self.bias_bit_width = bias_bit_width
         self.inter_bit_width = inter_bit_width
-        self.acti_bit_width = acti_bit_width
         self.retrain = retrain
         self.quant = quant
         if retrain is True:
             self.weight_log2_t = nn.Parameter(torch.Tensor(1))
-            self.acti_log2_t = nn.Parameter(torch.Tensor(1))
             if self.bias is not None:
                 self.bias_log2_t = nn.Parameter(torch.Tensor(1))
         else:
             self.weight_log2_t = torch.Tensor(1)
-            self.acti_log2_t = torch.Tensor(1)
             if self.bias is not None:
                 self.bias_log2_t = torch.Tensor(1)
 
@@ -48,9 +44,25 @@ class Linear(nn.Linear):
 
     def quantilize(self):
         self.quant = True
+        self.weight_log2_t.requires_grad = True
+        if self.bias is not None:
+            self.bias_log2_t.requires_grad = True
 
     def floatilize(self):
         self.quant = False
+        self.weight_log2_t.requires_grad = False
+        if self.bias is not None:
+            self.bias_log2_t.requires_grad = False
+
+    def quant_answer(self):
+        self.weight.data = qsigned(
+            self.weight, self.weight_log2_t,
+            self.weight_bit_width)**(self.weight_bit_width - 1 -
+                                     torch.ceil(self.weight_log2_t)).int()
+        self.bias.data = qsigned(
+            self.bias, self.bias_log2_t,
+            self.bias_bit_width)**(self.bias_bit_width - 1 -
+                                   torch.ceil(self.bias_log2_t)).int()
 
     def linear_forward(self, input):
         input_log2_t = input.abs().max().log2()
@@ -59,11 +71,11 @@ class Linear(nn.Linear):
         inter = qsigned(
             F.linear(input, weight, None),
             self.weight_log2_t + input_log2_t + math.log2(self.weight.numel()),
-            self.acti_bit_width)
+            self.inter_bit_width)
         if self.bias is not None:
             inter += qsigned(self.bias, self.bias_log2_t,
                              self.bias_bit_width).unsqueeze(0)
-        return qsigned(inter, self.acti_log2_t, self.acti_bit_width)
+        return inter
 
     def linear_forward_unquant(self, input):
         return F.linear(input, self.weight, self.bias)
